#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Core {
  public class Cloner : ICloner {
    private class ReferenceEqualityComparer : IEqualityComparer<ICloneable> {
      public bool Equals(ICloneable x, ICloneable y) {
        return object.ReferenceEquals(x, y);
      }

      public int GetHashCode(ICloneable obj) {
        if (obj == null) return 0;
        return obj.GetHashCode();
      }
    }

    private Dictionary<ICloneable, ICloneable> mapping;

    public Cloner() {
      mapping = new Dictionary<ICloneable, ICloneable>(new ReferenceEqualityComparer());
    }

    public ICloneable Clone(ICloneable obj) {
      if (obj == null) return null;
      ICloneable clone;
      if (mapping.TryGetValue(obj, out clone))
        return clone;
      else
        return obj.Clone(this);
    }
    public void AddClonedObject(ICloneable obj, ICloneable clone) {
      mapping.Add(obj, clone);
    }
  }
}
